--
with Bc.Containers;
with Bc.Containers.Collections;
with Bc.Containers.Collections.Bounded;
with Bc.Containers.Collections.Ordered;
with Bc.Containers.Collections.Ordered.Bounded;

with Ada.Text_IO;

package body Generic_Timed_Collection is
   use Ada;

   function "<" (L, R : Item_Type) return Boolean;

   package Items_Containers is new Bc.Containers (Item => Item_Type);
   package Items_Base_Collections is new Items_Containers.Collections;
   package Items_Collections is new Items_Base_Collections.Ordered;
   package Items_Cb is new Items_Collections.Bounded (Maximum_Size => Max_Nb_Of_Items);
   --
   --  Use type Timed_Items_Cb.Collection to operate on odered Timed_Item collections.

   -- -- --
   -- -- --

   protected Controller is
      procedure Add (Item : in Item_Type);
      procedure Purge (Item : in Item_Type);

      function Already_Handled (Item : in Item_Type) return Boolean;

      entry Get_Next_To_Purge (Item : out Item_Type);

   private
      The_Items : Items_Cb.Collection := Items_Cb.Null_Container;

      Item_Cb_Is_Empty : Boolean := True;
   end Controller;

   -- -- --

   protected body Controller is

      procedure Set_Empty_Status (Empty : Boolean := Items_Cb.Is_Empty (The_Items)) is
      begin

         if (Item_Cb_Is_Empty /= Empty) then
            Item_Cb_Is_Empty := Empty;
         end if;

      end Set_Empty_Status;

      -- -- --

      procedure Add (Item : in Item_Type) is
      begin

         if Items_Cb.Available (The_Items) > 0 then

            Items_Cb.Append (C => The_Items, Elem => Item);
            Set_Empty_Status (False);

         else
            raise No_Space_To_Store_Item;
         end if;
      end Add;

      -- -- --

      entry Get_Next_To_Purge (Item : out Item_Type)
         when (not Item_Cb_Is_Empty) is
      begin
         Item := Items_Cb.First (The_Items);
         --  Copy of oldest item
      end Get_Next_To_Purge;

      -- -- --

      function Already_Handled (Item : in Item_Type) return Boolean is
         It : Items_Containers.Iterator'class
            := Items_Cb.New_Iterator (The_Items);
      begin

         Iterate : while not Items_Containers.Is_Done (It) loop

            if Are_Equal (Items_Containers.Current_Item (It), Item) then
               return True;
            end if;

            Items_Containers.Next (It);

         end loop Iterate;

         return False;
      end Already_Handled;

      -- -- --

      procedure Purge (Item : in Item_Type) is
         At_Location : Natural;
      begin

         if (not Item_Cb_Is_Empty) then
            At_Location := Items_Cb.Location (C => The_Items, Elem => Item);

            if (At_Location > 0) then

               Items_Cb.Remove (C => The_Items, At_Index => At_Location);
               Set_Empty_Status;

            end if;

         end if;

      end Purge;

      -- -- --

   end Controller;

   -- -- --
   -- -- --

   task Timed_Purger is
   end Timed_Purger;

   -- -- --

   task body Timed_Purger is
      Item      : Item_Type;
      Next_Time : Calendar.Time;

      use type Calendar.Time;
   begin

      Text_IO.Put_Line ("#> Timed_Purger: Beginning ...");

      Purging : loop

         Text_IO.Put_Line ("#> Timed_Purger: Wait for item ...");

         Controller.Get_Next_To_Purge (Item);

         Next_Time := Get_Time_Stamp (Item) + Storage_Duration;

         Text_IO.Put_Line ("#> Timed_Purger: Delaying ...");

         delay until Next_Time;

         Ada.Text_IO.Put_Line ("#> Timed_Purger: Purging: " & Print (Item));

         Controller.Purge (Item);

         Ada.Text_IO.Put_Line ("#> Timed_Purger: Purged.");

      end loop Purging;

   end Timed_Purger;

   -- -- --

   function "<" (L, R : Item_Type) return Boolean is
      use type Calendar.Time;
   begin
      return (Get_Time_Stamp (L) < Get_Time_Stamp (R));
   end "<";

   -- -- --

   procedure Add (Item : in Item_Type) is
   begin
      Controller.Add (Item);
   end Add;

   -- -- --

   function Already_Handled (Item : in Item_Type) return Boolean is
   begin
      return Controller.Already_Handled (Item);
   end Already_Handled;

end Generic_Timed_Collection;
