--
with Generic_Timed_Collection;

with Ada.Calendar;
with Ada.Text_IO;
with Ada.Exceptions;

procedure Test_Timed_Collection is
   use Ada;

   type Test_Type is record
      Time_Stamp : Calendar.Time;
      Int        : Integer;
   end record;

   -- -- --

   function Are_Equal (L, R : Test_Type) return Boolean;
   function Are_Equal (L, R : Test_Type) return Boolean is
   begin
      return (L.Int = R.Int);
   end Are_Equal;

   -- -- --

   function Get_Time_Stamp (Item : Test_Type) return Ada.Calendar.Time;
   function Get_Time_Stamp (Item : Test_Type) return Ada.Calendar.Time is
   begin
      return Item.Time_Stamp;
   end Get_Time_Stamp;

   -- -- --

   function Print (Item : Test_Type) return String;
   function Print (Item : Test_Type) return String is
   begin
      return Integer'image (Item.Int);
   end Print;

   -- -- --

   Max_Nb_Of_Items  : constant Positive := 10;
   Storage_Duration : constant Duration := 5.0;

   More_Test_Data : constant Natural  := 0; --  10
   Test_Delay     : constant Duration := 1.0; --  0.0

   use type Calendar.Time;

   package Timed_Collection is new Generic_Timed_Collection (
      Item_Type        => Test_Type,
      Max_Nb_Of_Items  => Max_Nb_Of_Items,
      Storage_Duration => Storage_Duration);

   Test_Data : array (1 .. Max_Nb_Of_Items + More_Test_Data) of Test_Type;

begin

   Text_IO.Put_Line ("## test_timed_collection: Beginning ...");

   for Ix in Test_Data'range loop
      Text_IO.Put_Line ("#> Add ...");

      Test_Data (Ix) := (Calendar.Clock, Ix);

      if Timed_Collection.Already_Handled (Test_Data (Ix)) then
         Text_IO.Put_Line ("#> Already handled");
      else
         Text_IO.Put_Line ("#> Not handled");
      end if;

      Adding_Item : declare
      begin

         Timed_Collection.Add (Test_Data (Ix));

      exception
         when Timed_Collection.No_Space_To_Store_Item =>
            Text_IO.Put_Line ("#! No space to store item in collection");

         when others =>
            raise;
      end Adding_Item;

      if Timed_Collection.Already_Handled (Test_Data (Ix)) then
         Text_IO.Put_Line ("#> Already handled");
      else
         Text_IO.Put_Line ("#> Not handled");
      end if;

      delay Test_Delay;
   end loop;

   Text_IO.Put_Line ("## test_timed_collection: End.");

   Text_IO.Skip_Line;

exception
   when Event : others =>
      Text_IO.Put_Line ("#! test_timed_collection: EXCEPTION");
      Text_IO.Put_Line (Ada.Exceptions.Exception_Information (Event));
      Text_IO.Flush;

      Text_IO.Skip_Line;
end Test_Timed_Collection;
